/*
 * This program is to chagne the source of signals in SSGM.
 * If an external MCP signal were stopped, an internal MCP signal is driven.
 * And then, if the external signal were restarted, the external signal is driven again.
 *
 * At 23 Aug 2021, it was made by Jeongki Kim, SEEDCORE ltd. 
 */


#include <stdio.h>
#include <stdlib.h>
#include <string.h>		// strlen()
#include <unistd.h>		// read, write
#include <fcntl.h>		// open, close, O_RDWR, O_NONBLOCK
#include <sys/types.h>
#include <sys/ioctl.h>
#include <linux/types.h>
#include "my_ioctl.h"

#define SSGM_BASE_OFFSET	0x00000000	/* SSGM Base Address */
#define REGLOC_MAX_OFFSET	0x68		/* Maximum value of Offset in SSGM register */
#define REGLOC_MAIN_CTL		0x10		/* Register location of Main control */
#define REGLOC_EXTERNAL_MCP	0x60		/* Register location of External MCP */

#define CTLREG_INTERNAL		0x406		/* Control Register Value when Internal Signal */
#define CTLREG_EXTERNAL		0x20406		/* Control Register Value when External Signal */


unsigned int get_value_regloc( int fd, unsigned int regloc )
{
	SSGM_PARAM pi;

	if( regloc > REGLOC_MAX_OFFSET ) {
		printf( "Error: Over the range of register: 0x%08X.\n", regloc );
		return 0;
	}

	pi.addr = regloc;
	ioctl( fd, IOCTL_GET_SSGM_REG, &pi);
//	printf( "SSGM Reg: 0x%02X 0x%08X \n", pi.addr, pi.val );

	return pi.val;
}


int set_value_ctlreg( int fd, unsigned int regval )
{
	SSGM_PARAM pi;

	if( (regval & 1) > 0 ) {
		printf( "Error: Value(0x%08X) is not suitable.\n", regval );
		return -1;
	}

	pi.addr = REGLOC_MAIN_CTL;
	pi.val = 1;			// to reset the control register.
	ioctl( fd, IOCTL_SET_SSGM_REG, &pi);

	pi.val = regval;
	ioctl( fd, IOCTL_SET_SSGM_REG, &pi);
	return 0;
}


int change_external_interal_signals( int fd )
{
	unsigned int extern_cnt1, extern_cnt2;
	unsigned int ctlreg_val;

	extern_cnt1 = get_value_regloc( fd, REGLOC_EXTERNAL_MCP );

	while(1) {
		sleep(2);

		ctlreg_val  = get_value_regloc( fd, REGLOC_MAIN_CTL );
		extern_cnt2 = get_value_regloc( fd, REGLOC_EXTERNAL_MCP );

		if( (ctlreg_val & 0x20000) > 0 ) {		// external signal.
			if( extern_cnt1 == extern_cnt2 ) {	// change to internal. 
				set_value_ctlreg( fd, CTLREG_INTERNAL );
				printf( "Now SSGM generates the Internal signals(0x%X).\n", CTLREG_INTERNAL );
			}
			extern_cnt1 = extern_cnt2;
		}
		else {						// internal signal.
			if( extern_cnt1 != extern_cnt2 ) {	// change to external. 
				set_value_ctlreg( fd, CTLREG_EXTERNAL );
				printf( "Now SSGM generates the External signals(0x%X).\n", CTLREG_EXTERNAL );
			}
			extern_cnt1 = extern_cnt2;
		}

	}
}


int main( int argc, char **argv )
{
	int fd;
	char devfilename[200];

	sprintf( devfilename, "/dev/scSSGM" );
	fd = open( devfilename, O_RDWR|O_SYNC );
	if( !(0<fd) ) {
		printf( "Devide file(%s) open fail.\n", devfilename );
		return -1; 
	}
	printf( "**SSGM device file(%s) open ok!\n\n", devfilename );

	change_external_interal_signals( fd );

	close(fd);
	printf("**close()\n");
	return 0;
}

