/*
 * This is functions to read register values. 
 */
#include <stdio.h>
#include <string.h>		// strlen()
#include <unistd.h>		// read, write
#include <fcntl.h>		// open, close, O_RDWR, O_NONBLOCK
#include <sys/types.h>
#include <sys/ioctl.h>
#include <linux/types.h>
#include "my_ioctl.h"

#define SSGM_BASE_OFFSET	0x00000000	/* SSGM Base Address */
#define NUM_SSGM_REG_VALUE	27		/* Number of SSGM Register Values */


static char *reg_desc[] = {
	"Production Rev.",
	"Build Date.",
	"Serial Number.",
	"Timer Register.",
	"Control Register.\n\tB(0) 1:Reset, 0:Normal Op.\n\tB(1) 1:PCP Contin Mode.\n\tB(2) 1:PCP start.\n\tB(8) 1:MCP/PCP Self-Gen mode.\n\tB(17:16) 10:Pulse Extern Sel.\n\tB(19:18) 10:CLK Extern Sel.",
	"Interrupt Mask: Not used.",
	"Interrupt Status: Not Used.",
	"Pluse Enable Reg.",
	"Pluse Output Source Selection.",
	"MCP Tick Duration: 0x7080 =600usec.",
	"PCP Tick Duration: 600usec.",
	"Pulse High Time: 1us = 0x30/48MHz.",
	"NCP Delay: 0xE100 =1200usec.",
	"SOP1 Delay.",
	"SOP2 Delay.",
	"SOP3 Delay.",
	"MCP Counter.",		// 0x40
	"NCP Counter.",
	"PCP Counter.",
	"SOP1 Counter.",
	"SOP2 Counter.",
	"SOP3 Counter.",
	"Number of PCP to Create.",
	"PCP Delay from Source.",
	"External MCP Counter.",	// 0x60
	"External PCP Counter.",
	"External NCP Counter."
};


int main()
{
	int fd;
	char devfilename[200];
	SSGM_PARAM pi;
	int i;

	sprintf( devfilename, "/dev/scSSGM" );
	fd = open( devfilename, O_RDWR|O_SYNC );
	if( !(0<fd) ) {
		printf( "Devide file(%s) open fail.\n", devfilename );
		return -1; 
	}

	printf( "**device file(%s) open ok!\n\n", devfilename );
	printf( "   Offset  Value     Description \n" );
	printf( "---------------------------------------\n" );

	for( i=0; i<NUM_SSGM_REG_VALUE; i++) {
		pi.addr = SSGM_BASE_OFFSET + (i*4);
		ioctl(fd, IOCTL_GET_SSGM_REG, &pi);
		printf( "SSGM 0x%02X 0x%08X %s\n", i*4, pi.val, reg_desc[i] );
	}

	close(fd);
	printf("**close()\n");
	return 0;
}

