/*
 * This is functions to set register values of SSGM. 
 */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>		// strlen()
#include <unistd.h>		// read, write
#include <fcntl.h>		// open, close, O_RDWR, O_NONBLOCK
#include <sys/types.h>
#include <sys/ioctl.h>
#include <linux/types.h>
#include "my_ioctl.h"

#define SSGM_BASE_OFFSET	0x00000000	/* SSGM Base Address */
#define MAX_OFFSET_NUM		0x68		/* Maximum value of Offset in SSGM register */


static char *reg_desc[] = {
	"Production Rev.",
	"Build Date.",
	"Serial Number.",
	"Timer Register.",
	"Control Register.\n\tB(0) 1:Reset, 0:Normal Op.\n\tB(1) 1:PCP Contin Mode.\n\tB(2) 1:PCP start.\n\tB(8) 1:MCP/PCP Self-Gen mode.\n\tB(17:16) 10:Pulse Extern Sel.\n\tB(19:18) 10:CLK Extern Sel.",
	"Interrupt Mask: Not used.",
	"Interrupt Status: Not Used.",
	"Pluse Enable Reg.",
	"Pluse Output Source Selection.",
	"MCP Tick Duration: 0x7080 =600usec.",
	"PCP Tick Duration: 600usec.",
	"Pulse High Time: 1us = 0x30/48MHz.",
	"NCP Delay: 0xE100 =1200usec.",
	"SOP1 Delay.",
	"SOP2 Delay.",
	"SOP3 Delay.",
	"MCP Counter.",
	"PCP Counter.",
	"NCP Counter.",
	"SOP1 Counter.",
	"SOP2 Counter.",
	"SOP3 Counter.",
	"Number of PCP to Create.",
	"PCP Delay from Source.",
	"External MCP Counter.",	// 0x60
	"External PCP Counter.",
	"External NCP Counter."
};


int get_main_args( SSGM_PARAM *pi, int argc, char **argv )
{
	char rstr[128];

	if( argc != 3 ) printf( "Usage: %s Offset(Hex) Value(HEX)\n\n", argv[0] );
	if( argc > 3 ) return -1;

	while(1) {
		if( argc == 1 ) {
			printf( "Select an Offset(Hex) and Value(Hex) to set..\n" ); 
			printf( "Range of the Offset is from 0x00 to 0x%02X: ", MAX_OFFSET_NUM );
			scanf( "%s", rstr );
			pi->addr = strtol( rstr, NULL, 16 );

			printf( "Value(Hex) : " );
			scanf( "%s", rstr );
			pi->val = strtol( rstr, NULL, 16 );
		}
		else if( argc == 2 ) {
			pi->addr = strtol( argv[1], NULL, 16 );
			printf( "Value(HEX): " );
			scanf( "%s", rstr );
			pi->val = strtol( rstr, NULL, 16 );
		}
		else if( argc == 3 ) {
			pi->addr = strtol( argv[1], NULL, 16 );
			pi->val = strtol( argv[2], NULL, 16 );
		}

		if( pi->addr <0 || (pi->addr > MAX_OFFSET_NUM) ) {
			printf( "Range of the data is over: 0x%02X.\n", pi->addr );
			argc =1;
			continue;
		}
		else break;
	}
	return 0;
}


int main( int argc, char **argv )
{
	int fd;
	char devfilename[200];
	SSGM_PARAM pi;
	int i;

	if( get_main_args( &pi, argc, argv ) )	return -1;

	sprintf( devfilename, "/dev/scSSGM" );
	fd = open( devfilename, O_RDWR|O_SYNC );
	if( !(0<fd) ) {
		printf( "Devide file(%s) open fail.\n", devfilename );
		return -1; 
	}
	printf( "**SSGM device file(%s) open ok!\n\n", devfilename );

	printf( "SSGM Reg: 0x%02X 0x%08X was written.\n", pi.addr, pi.val );
	ioctl(fd, IOCTL_SET_SSGM_REG, &pi);

	printf( "---------------------------------------\n" );
	sleep(1);

	ioctl(fd, IOCTL_GET_SSGM_REG, &pi);
	printf( "Now SSGM Reg: 0x%02X 0x%08X %s\n", pi.addr, pi.val, reg_desc[pi.addr/4] );

	close(fd);
	printf("**close()\n");
	return 0;
}

